import React from 'react';
import { StyleSheet, View } from 'react-native';
import { observer } from 'mobx-react';
import PropTypes from 'prop-types';
import Background from '../component/background';
import MainContent from '../component/main-content';
import { Header, Title } from '../component/header';
import { Alert } from '../component/notification';
import { color } from '../component/style';
import { H4Text } from '../component/text';
import Icon from '../component/icon';
import ChannelIcon from '../asset/icon/channel';
import LightningBoltPurpleIcon from '../asset/icon/lightning-bolt-purple';
import {
  BalanceLabel,
  BalanceLabelNumeral,
  BalanceLabelUnit,
} from '../component/label';
import { Button, QrButton, GlasButton, DownButton } from '../component/button';

//
// Home View
//

const styles = StyleSheet.create({
  content: {
    paddingLeft: 20,
    paddingRight: 20,
  },
  downBtn: {
    margin: 25,
  },
});

const HomeView = ({
  store,
  wallet,
  channel,
  payment,
  invoice,
  transaction,
  nav,
}) => {
  const {
    totalBalanceLabel,
    unitLabel,
    channelStatus,
    channelPercentageLabel,
  } = store;
  return (
    <Background image="purple-gradient-bg">
      <HomeHeader
        goChannels={() => channel.init()}
        goSettings={() => nav.goSettings()}
        showChannelAlert={store.showChannelAlert}
      />
      <QrCodeSeparator goDeposit={() => nav.goDeposit()} />
      <MainContent style={styles.content}>
        <BalanceDisplay
          totalBalanceLabel={totalBalanceLabel}
          unitLabel={unitLabel}
          channelStatus={channelStatus}
          channelPercentageLabel={channelPercentageLabel}
          toggleDisplayFiat={() => wallet.toggleDisplayFiat()}
        />
        <SendReceiveButton
          goPay={() => payment.init()}
          goRequest={() => invoice.init()}
        />
        <DownButton onPress={() => transaction.init()} style={styles.downBtn}>
          Transactions
        </DownButton>
      </MainContent>
    </Background>
  );
};

HomeView.propTypes = {
  store: PropTypes.object.isRequired,
  wallet: PropTypes.object.isRequired,
  channel: PropTypes.object.isRequired,
  payment: PropTypes.object.isRequired,
  invoice: PropTypes.object.isRequired,
  transaction: PropTypes.object.isRequired,
  nav: PropTypes.object.isRequired,
};

//
// Balance Display
//

const balanceStyles = StyleSheet.create({
  wrapper: {
    flex: 1,
    justifyContent: 'center',
  },
  percentWrapper: {
    flexDirection: 'row',
    alignItems: 'center',
    marginTop: 20,
    marginBottom: 5,
  },
  alert: {
    marginRight: 6,
  },
});

const BalanceDisplay = ({
  totalBalanceLabel,
  unitLabel,
  channelStatus,
  channelPercentageLabel,
  toggleDisplayFiat,
}) => (
  <View style={balanceStyles.wrapper}>
    <Button onPress={toggleDisplayFiat}>
      <BalanceLabel>
        <BalanceLabelNumeral>{totalBalanceLabel}</BalanceLabelNumeral>
        <BalanceLabelUnit>{unitLabel}</BalanceLabelUnit>
      </BalanceLabel>
      <View style={balanceStyles.percentWrapper}>
        <Alert type={channelStatus} style={balanceStyles.alert} />
        <H4Text>{channelPercentageLabel}</H4Text>
      </View>
    </Button>
  </View>
);

BalanceDisplay.propTypes = {
  totalBalanceLabel: PropTypes.string.isRequired,
  unitLabel: PropTypes.string,
  channelStatus: PropTypes.string.isRequired,
  channelPercentageLabel: PropTypes.string.isRequired,
  toggleDisplayFiat: PropTypes.func.isRequired,
};

//
// Send Receive Button
//

const bigBtnStyles = StyleSheet.create({
  wrapper: {
    alignSelf: 'stretch',
    flexDirection: 'row',
  },
  leftBtn: {
    flex: 1,
    borderTopLeftRadius: 21,
    borderBottomLeftRadius: 21,
  },
  rightBtn: {
    flex: 1,
    borderTopRightRadius: 21,
    borderBottomRightRadius: 21,
  },
  boltWrapper: {
    justifyContent: 'center',
    backgroundColor: color.glas,
  },
});

const SendReceiveButton = ({ goPay, goRequest }) => (
  <View style={bigBtnStyles.wrapper}>
    <GlasButton onPress={goRequest} style={bigBtnStyles.leftBtn}>
      Request
    </GlasButton>
    <View style={bigBtnStyles.boltWrapper}>
      <LightningBoltPurpleIcon height={126 * 0.4} width={64 * 0.4} />
    </View>
    <GlasButton onPress={goPay} style={bigBtnStyles.rightBtn}>
      Pay
    </GlasButton>
  </View>
);

SendReceiveButton.propTypes = {
  goPay: PropTypes.func.isRequired,
  goRequest: PropTypes.func.isRequired,
};

//
// Home Header
//

const headerStyles = StyleSheet.create({
  btnWrapperLeft: {
    width: 150,
    flexDirection: 'row',
    justifyContent: 'flex-start',
  },
  btnWrapperRight: {
    width: 150,
    flexDirection: 'row',
    justifyContent: 'flex-end',
  },
  channelBtn: {
    marginLeft: 3,
  },
  channelAlert: {
    position: 'absolute',
    top: 17,
    right: 21,
    height: 10,
    width: 10,
    backgroundColor: color.pinkSig,
    borderColor: color.white,
    borderRadius: 50,
    borderStyle: 'solid',
    borderWidth: 2,
  },
  settingsBtn: {
    marginRight: 3,
  },
  settingsIcon: {
    height: 22,
    width: 22,
  },
});

const HomeHeader = ({ goChannels, goSettings, showChannelAlert }) => (
  <Header>
    <Button onPress={goChannels} style={headerStyles.channelBtn}>
      <ChannelIcon height={24 * 0.9} width={25 * 0.9} />
      {showChannelAlert ? <View style={headerStyles.channelAlert} /> : null}
    </Button>
    <Title title="Wallet" />
    <Button onPress={goSettings} style={headerStyles.settingsBtn}>
      <Icon
        image={require('../asset/icon/settings.png')}
        style={headerStyles.settingsIcon}
      />
    </Button>
  </Header>
);

HomeHeader.propTypes = {
  goChannels: PropTypes.func.isRequired,
  goSettings: PropTypes.func.isRequired,
  showChannelAlert: PropTypes.bool.isRequired,
};

//
// QR Code Separator
//

const qrStyles = StyleSheet.create({
  wrapper: {
    alignSelf: 'stretch',
    flexDirection: 'row',
    alignItems: 'flex-start',
    height: 30,
    top: -1,
  },
  separator: {
    flex: 1,
    height: 1,
    shadowOffset: { width: 0, height: 0.25 },
    shadowColor: color.white,
  },
  button: {
    top: -19,
  },
});

const QrCodeSeparator = ({ goDeposit }) => (
  <View style={qrStyles.wrapper}>
    <View style={qrStyles.separator} />
    <QrButton onPress={goDeposit} style={qrStyles.button}>
      Add coin
    </QrButton>
    <View style={qrStyles.separator} />
  </View>
);

QrCodeSeparator.propTypes = {
  goDeposit: PropTypes.func.isRequired,
};

export default observer(HomeView);
